require 'puppet/face'
require 'puppet/cloudpack/vmware'

Puppet::Face.define(:node_vmware, '0.0.1') do
  action :list do
    summary "List VMware virtual machines"
    description <<-'EOT'
      List VMware virtual machines in all Datacenters.  This command may take
      considerable time to run in large VMware environments.

      The default output of the list action provides a limited set of
      information.  To obtain all available attributes for each virtual
      machine, please use a different rendering mode, such as --render-as yaml.
    EOT

    returns 'Array of virtual machine attribute hashes.'

    examples <<-'EOT'
      List all VMs in the entire inventory:

          $ puppet node_vmware list
          /Datacenters/Solutions/vm/centos5-01
            powerstate: poweredOff
            name:       centos5-01
            hostname:   centos5.example.vm
            instanceid: 503271e5-2cc4-1f1a-4303-75d33938fcf4
            ipaddress:  ---.---.---.---
            template:   false

          /Datacenters/Solutions/vm/el56
            powerstate: poweredOff
            name:       el56
            hostname:   --------
            instanceid: 50323f93-6835-1178-8b8f-9e2109890e1a
            ipaddress:  ---.---.---.---
            template:   true

      List only the VMs in the Templates folder:

          $ puppet node_vmware list --folder=/Datacenters/DC1/vm/Templates
          /Datacenters/Solutions/vm/Templates/el56template
            powerstate: poweredOff
            name:       el56template
            hostname:   --------
            instanceid: 09CB9D9B-2498-41E0-807A-382408829C07
            ipaddress:  ---.---.---.---
            template:   true
    EOT

    option '--folder=', '-f=' do
      summary 'Limit the listing to a single folder.'
      description <<-'EOT'
        This option limits the listing of virtual machines to a single folder, which
        is much faster than listing all virtual machines in the entire inventory.
      EOT
    end

    Puppet::CloudPack::VMware.add_timeout_option(self)

    when_invoked do |options|
      Puppet::CloudPack::VMware.list(options)
    end

    when_rendering :console do |return_value|
      Puppet.notice "Complete"
      result_map = {}
      return_value.each do |vm|
        result_map["#{vm[:path]}/#{vm[:name]}"] = vm
      end
      memo = ''
      result_map.keys.sort { |l, r| l.downcase <=> r.downcase }.each do |full_path|
        vm = result_map[full_path]

        # Add a blank line if this isn't the first record
        memo << "\n" unless memo.empty?

        # Add the next record string
        memo << full_path << "\n" <<
          "  powerstate: #{vm[:power_state]}\n" <<
          "  name:       #{vm[:name]}\n" <<
          "  hostname:   #{vm[:hostname] || '--------'}\n" <<
          "  instanceid: #{vm[:instance_uuid]}\n" <<
          "  ipaddress:  #{vm[:ipaddress] || '---.---.---.---'}\n" <<
          "  template:   #{vm[:is_a_template]}\n"
      end
      memo
    end
  end
end
